
#include <netlink/netlink.h>
#include <netlink/types.h>
#include <netlink/utils.h>
#include <netlink/msg.h>

#include "isniffy.h"
#include "config.h"


static int fsm_new(struct sf_config *cfg, struct sf_terminal *term, struct nl_msg *msg)
{
	struct sniffy_msg_group *gmsg;
	struct nlmsghdr *hdr;

	hdr = nlmsg_hdr(msg);
	gmsg = nlmsg_data(hdr);

	switch(hdr->nlmsg_type) {
	case SNIFFY_CMD_GROUP:
		SF_LOG_PRINT2(cfg, "got group %u for idx %d\n", gmsg->group, gmsg->cmsg.idx);

		if (nl_socket_add_membership(cfg->nlh, gmsg->group) < 0) {
			printf("nl_socket_add_membership failed: %s\n", nl_geterror());
			term->state = SF_STATE_DONE;
			break;
		}

		term->state = SF_STATE_DATA;
		break;
	default:
		printf("failed pty %d: got unexpected msg type %d for state NEW\n", 
			term->idx, hdr->nlmsg_type);
	}
	
	return 0;
}

static int fsm_data(struct sf_config *cfg, struct sf_terminal *term, struct nl_msg *msg)
{
	struct sniffy_msg_data *dmsg;
	struct nlmsghdr *hdr;

	hdr = nlmsg_hdr(msg);
	dmsg = nlmsg_data(hdr);

	switch(hdr->nlmsg_type) {
	case SNIFFY_CMD_DATA:
		sniffy_process(cfg, term, dmsg);
		break;

	case SNIFFY_CMD_DETACH:
		term->state = SF_STATE_DONE;
		break;
	}

	return 0;
}

int sf_fsm_terminal_process(struct sf_config *cfg, struct nl_msg *msg)
{
	struct sniffy_msg_common *cmsg;
	struct nlmsghdr *hdr;
	struct sf_terminal *term;

	hdr = nlmsg_hdr(msg);
	cmsg = nlmsg_data(hdr);

	if (NULL == (term = sf_config_terminal_find(cfg, cmsg->idx))) {
		printf("failed: terminal pty%d not found\n", cmsg->idx);
		return -1;
	}

	switch(term->state) {
	case SF_STATE_NEW:
		fsm_new(cfg, term, msg);
		break;

	case SF_STATE_DATA:
		fsm_data(cfg, term, msg);
		break;
	}

	/* PTY is closed from kernel, we dont need 
	   to detach by ourselfs */
	if (SF_STATE_DONE == term->state)
		sf_term_del(cfg, term, 0);

	return 0;
}
