
#include <stdio.h>
#include <stdarg.h>
#include <utmp.h>

#include "config.h"

#define SNIFFY_PROMPT     "\n>>> sniffy "
#define SNIFFY_LIST       SNIFFY_PROMPT"- pty list: \n"
#define SNIFFY_WATCH      SNIFFY_PROMPT"- watch changed: \n"
#define SNIFFY_ERROR      SNIFFY_PROMPT"- error: "
#define SNIFFY_SWITCH     SNIFFY_PROMPT"- switched to: \n"
#define SNIFFY_CHANGE     SNIFFY_PROMPT"- change pty to: \n"
#define SNIFFY_HELP       SNIFFY_PROMPT"- help: \n"

extern int exit_fl;

static struct utmp* get_utmp(int idx)
{
	struct utmp *u = NULL;

	while((u = getutent())) {
		int u_idx;
		if (1 != sscanf(u->ut_line, "pts/%d", &u_idx))
			continue;

		if (u_idx == idx)
			break;
	}

	return u;
}

static int display_terminal(struct sf_config *cfg, struct sf_terminal *term, int i, int first)
{
	char *display  = "none";
	char *trace    = "none";
	char *ut_user  = "none";
	char *ut_host  = "none";
	unsigned int trace_size = 0;
	struct utmp *u;

	if (term->display_slave)
		display = "slave";

	if (term->display_master)
		display = "master";

	if ((term->display_slave) &&
	    (term->display_master))
		display = "slave master";

	if (term->trace) {
		trace = term->trace_filename;
		trace_size = term->trace_size - TRACE_MMAP_SIZE + (term->trace_mmap_cur - term->trace_mmap_ptr);
	}

	setutent();

	if (NULL != (u = get_utmp(term->idx))) {
		ut_user = u->ut_user;
		ut_host = u->ut_host;
	}

	endutent();

	if (first)
		fprintf(cfg->tty_out, "%4s %3s %3s %10s %-10s %10s %14s %s\n",
		"", "cur", "idx", "user", "host", "rx bytes", "display", "trace");

	fprintf(cfg->tty_out, "%3d. %3s %3d %10s@%-10s %10u %14s %s (%u)\n",
		i,
		term == cfg->terminal_current ? "*" : " ",
		term->idx,
		ut_user,
		ut_host,
		term->rx_bytes,
		display,
		trace,
		trace_size
	);

	return 0;
}

static int change_terminal_idx_blk = 0;
static int change_terminal_idx_displ_max = 10;
static struct sf_terminal *change_terminal_current;

static int cmd_change_terminal(struct sf_config *cfg, int c)
{
	static int display_master;
	static int display_slave;

	/* change currently displayed pty */
	if (c == 'c') {
		struct sf_terminal *term;
		int i;

		cfg->input_change_terminal = 1;
		change_terminal_idx_blk = 0;

		display_master = cfg->terminal_current->display_master;
		display_slave = cfg->terminal_current->display_slave;
		cfg->terminal_current->display_master = 0;
		cfg->terminal_current->display_slave = 0;

		fprintf(cfg->tty_out, SNIFFY_CHANGE);
		for(i = 0, term = cfg->terminal; 
		    term != NULL && i < change_terminal_idx_displ_max; 
		    term = term->next, i++) {

			display_terminal(cfg, term, i, i == 0);
		}

		change_terminal_current = term;

		fprintf(cfg->tty_out, SNIFFY_PROMPT);
		if (change_terminal_current)
			fprintf(cfg->tty_out, "press <space> for more, or ");

		fprintf(cfg->tty_out, "choose %d - %d\n", 0, i - 1);

		return 0;
	}

	if (c == ' ') {
		struct sf_terminal *term;
		int i;

		if (!change_terminal_current)
			return 0;

		change_terminal_idx_blk++;

		for(i = 0, term = change_terminal_current;
		    term != NULL && i < change_terminal_idx_displ_max; 
		    term = term->next, i++) {

			display_terminal(cfg, term, i, i == 0);
		}

		change_terminal_current = term;

		fprintf(cfg->tty_out, SNIFFY_PROMPT);
		if (change_terminal_current)
			fprintf(cfg->tty_out, "press <space> for more, or ");

		fprintf(cfg->tty_out, "choose %d - %d\n", 0, i - 1);

		change_terminal_current = term;
		return 0;
	}

	if (cfg->input_change_terminal) {
		int i = c - '0';
		struct sf_terminal *term;
		int i1 = i;

		i += change_terminal_idx_blk * change_terminal_idx_displ_max;

		for(term = cfg->terminal; (term != NULL) && (i); term = term->next, i--);

		cfg->terminal_current->display_master = display_master;
		cfg->terminal_current->display_slave = display_slave;

		if (term) {
			cfg->terminal_current = term;
			cfg->terminal_current_num = i1;
			fprintf(cfg->tty_out, SNIFFY_SWITCH);
			display_terminal(cfg, term, i1, 1);
		}
		else {
			fprintf(cfg->tty_out, SNIFFY_ERROR"wrong pty number, change failed\n");
			cfg->input_change_terminal = 0;
			return 0;
		}

		cfg->input_change_terminal = 0;
	}
	return 0;
}

int sf_input_cmd_process(struct sf_config *cfg, int c)
{
	if (!cfg->input_change_terminal) {

		if (cfg->terminal_current) {
			/* change master slave visibility */
			if (c == 's')
				cfg->terminal_current->display_slave = !cfg->terminal_current->display_slave;

			if (c == 'm')
				cfg->terminal_current->display_master = !cfg->terminal_current->display_master;

			if (c == 's' || c == 'm') {
				fprintf(cfg->tty_out, SNIFFY_WATCH);
				display_terminal(cfg, cfg->terminal_current, cfg->terminal_current_num, 1);
				return 0;
			}
		}

		/* list currenttly attached ptys */
		if (c == 'l') {
			struct sf_terminal *term;
			int i;

			fprintf(cfg->tty_out, SNIFFY_LIST);
			for(i = 0, term = cfg->terminal; term != NULL; term = term->next, i++) {
				display_terminal(cfg, term, i, i == 0);
			}

			return 0;
		}

		if (c == 'h') {
			fprintf(cfg->tty_out, SNIFFY_HELP);
			fprintf(cfg->tty_out, "c[0-9] - change current pty displayed\n");
			fprintf(cfg->tty_out, "l      - list all watching ptys\n");
			fprintf(cfg->tty_out, "s      - togle slave  pty display\n");
			fprintf(cfg->tty_out, "m      - togle master pty display\n");
			fprintf(cfg->tty_out, "q      - shutdown the server\n");
			fprintf(cfg->tty_out, "^C     - client mode - shutdown the client\n");
			fprintf(cfg->tty_out, "\n");

			return 0;
		}
	}

	if ((c == 'c') || (cfg->input_change_terminal))
		cmd_change_terminal(cfg, c);

	if (c == 'q') {
		exit_fl = 1;
		fprintf(cfg->tty_out, SNIFFY_PROMPT "going down...\n");
		return 0;
	}

	return 0;
}

int sf_input_cmd_print(struct sf_config *cfg, char *str, va_list ap)
{
	fprintf(cfg->tty_out, SNIFFY_PROMPT);
	vfprintf(cfg->tty_out, str, ap);
	return 0;
}
