
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include <netlink/netlink.h>
#include <netlink/types.h>
#include <netlink/utils.h>
#include <netlink/msg.h>

#include "isniffy.h"
#include "config.h"

static struct sf_config *scfg = NULL;

static int recv_cb(struct nl_msg *msg, void *arg)
{
	sf_fsm_terminal_process(scfg, msg);

	/* allways want whatever comes next :) */
	return NL_OK;
}

int sf_intf_attach(struct sf_config *cfg, struct sf_terminal *term, int detach)
{
	struct sniffy_msg_common cmsg;
	struct nl_msg *msg;
	int ret;

	msg = nlmsg_alloc_simple(detach ? SNIFFY_CMD_DETACH : SNIFFY_CMD_ATTACH, 0);
	cmsg.idx = term->idx;
	nlmsg_append(msg, &cmsg, sizeof(cmsg), 0);

	if ((ret = nl_send(cfg->nlh, msg)) < 0) {
		printf("nl_send failed: %s\n", nl_geterror());
		return -1;
	}

	return 0;
}

static int control_connection()
{
	FILE *f;
	int minor = -1;

	if (NULL == (f = fopen("/proc/misc", "r"))) {
		perror("fopen failed");
		return -1;
	}

	while (!feof(f)) {
		char line[100];
		char dev[100];

		if (!fgets(line, sizeof(line), f))
			break;
		
		if (2 != sscanf(line, "%d %s\n", &minor, dev)) {
			perror("sscanf failed");
			break;
		}

		if (!strcmp(SNIFFY_DEV, dev))
			break;
	}

	fclose(f);

	if (-1 == minor)
		return -1;

#define MKDEV(ma,mi)    ((ma)<<8 | (mi))

	if (((-1 == mknod("/dev/"SNIFFY_DEV, 
				O_RDONLY | S_IFCHR, 
				MKDEV(10, minor)))) && 
	    (EEXIST != errno)) {
		perror("mknod failed");
		return -1;
	}
#undef MKDEV

	if (-1 == open("/dev/"SNIFFY_DEV, O_RDONLY)) {
		perror("mknod failed");
		return -1;
	}
		
	return 0;
}

int sf_intf_fd(struct sf_config *cfg)
{
	return nl_socket_get_fd(cfg->nlh);
}

int sf_intf_init(struct sf_config *cfg)
{
	if (-1 == control_connection())
		return -1;

	/* setup netlink connection */
	cfg->nlh = nl_handle_alloc();

	if (nl_connect(cfg->nlh, NETLINK_SNIFFY)) {
		printf("nl_connect failed: %s\n", nl_geterror());
		return -1;
	}

	nl_cb_set(nl_socket_get_cb(cfg->nlh), NL_CB_MSG_IN, NL_CB_CUSTOM, recv_cb, NULL);
	nl_socket_set_nonblocking(cfg->nlh);

	return 0;
}

int sf_intf_close(struct sf_config *cfg)
{
	nl_handle_destroy(cfg->nlh);
	return 0;
}

int sf_intf_recv(struct sf_config *cfg)
{
	scfg = cfg;
	return nl_recvmsgs_default(cfg->nlh);
}
