
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/inotify.h>

#include "config.h"


int sf_notify_init(struct sf_config *cfg)
{
	if (-1 == (cfg->inotify_fd = inotify_init())) {
		perror("inotify_init failed");
		return -1;
	}

	/* watch ptys */
	if (-1 == (cfg->inotify_wd_pts = inotify_add_watch(cfg->inotify_fd, 
					SF_DEVPTS_PATH,
					IN_CREATE))) {
		perror("failed inotify_add_watch");
		return -1;
	}

	return 0;
}

int sf_notify_close(struct sf_config *cfg)
{
	return close(cfg->inotify_fd);
}

int sf_notify_process(struct sf_config *cfg)
{
#define MAXDATA 4096
	char notify_data[MAXDATA];
	struct inotify_event *event = (struct inotify_event*) notify_data;
	size_t len;
	int idx;
	struct sf_terminal *term;

	if (-1 == (len = read(cfg->inotify_fd, event, MAXDATA)))
                return -1;

	if (event->wd != cfg->inotify_wd_pts)
		return -1;

	if ((idx = strtol(event->name, (char **) NULL, 10)) < 0)
		return -1;

	/* We dont need to look for IN_DELETE, since the kernel module 
	   should take care about it and notify us with 
	   the SNIFFY_CMD_DETACH message */
	if (event->mask & IN_CREATE) {

		if (NULL == (term = sf_config_terminal_add(cfg, idx)))
			return -1;

		return sf_term_add(cfg, term);
	}

	return 0;
}
