
#include <X11/X.h>
#include <X11/cursorfont.h>
#include <X11/Xmu/WinUtil.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "config.h"


#define SelectButtonAny (-1)

static XID get_window_id(struct sf_config *cfg)
{
	char ctlseq[] = "\033[8;20;120t";
	char *wid;
	XID id;
	char *endptr;

	write(1, ctlseq, strlen(ctlseq));

	if (NULL == (wid = getenv("WINDOWID"))) {
		SF_LOG_PRINT1(cfg, "failed to get window id via $WINDOWID variable\n");
		return None;
	}

	SF_LOG_PRINT2(cfg, "getting window id from $WINDOWID variable: %s\n", wid);

	id = strtol(wid, &endptr, 10);
	if (*endptr) {
		SF_LOG_PRINT2(cfg, "failed to get valid window id from $WINDOWID variable: %s\n", wid);
		return None;
	}

	return id;
}

/* get_window_id - get the X11 window id

   TODO stolen from xkill, notice the cursor shape :)
	need a non interactive way to find the window id,
	now user interaction to click on our terminal 
	window is needed 
*/

static XID get_window_id_click(Display *dpy, int screen, int button)
{
	Cursor cursor;
	Window root;
	Window retwin = None;
	int retbutton = -1;
	int pressed = 0;

#define MASK (ButtonPressMask | ButtonReleaseMask)

	root = RootWindow (dpy, screen);
	cursor = XCreateFontCursor(dpy, XC_pirate);
	if (cursor == None) {
		printf("failed to create selection cursor\n");
		return -1;
	}

	printf("Select window to apply resize,... sorry :)\n");

	XSync(dpy, 0);

	if (XGrabPointer(dpy, root, False, MASK, GrabModeSync, GrabModeAsync,
			None, cursor, CurrentTime) != GrabSuccess) {
		printf("Failed to grab cursor\n");
		return -1;
	}

	/* from dsimple.c in xwininfo */
	while (retwin == None || pressed != 0) {
		XEvent event;

		XAllowEvents(dpy, SyncPointer, CurrentTime);
		XWindowEvent(dpy, root, MASK, &event);

		switch (event.type) {
		case ButtonPress:
			if (retwin == None) {
				retbutton = event.xbutton.button;
				retwin = ((event.xbutton.subwindow != None) ?
				event.xbutton.subwindow : root);
			}
			pressed++;
			continue;
		case ButtonRelease:
			if (pressed > 0) pressed--;
			continue;
		}
	}

	XUngrabPointer(dpy, CurrentTime);
	XFreeCursor(dpy, cursor);
	XSync(dpy, 0);

	return ((button == -1 || retbutton == button) ? retwin : None);
}

int sf_resize_init(struct sf_config *cfg)
{
	int screenno;
	int button;
	XID id = None;

	SF_LOG_PRINT1(cfg, "initialize, getting the window\n");

	if (NULL == (cfg->display = XOpenDisplay(NULL))) {
		printf ("Unable to open display \"%s\"\n", XDisplayName (NULL));
		return -1;
	}

	if (None == (id = get_window_id(cfg))) {
	
		screenno = DefaultScreen(cfg->display);
		button = SelectButtonAny;

		if (None != (id = get_window_id_click(cfg->display, screenno, button))) {
			if (id == RootWindow(cfg->display, screenno)) 
				id = None;

			id = XmuClientWindow(cfg->display, id);
		}

		if (None == id)
			return -1;
	}

	SF_LOG_PRINT2(cfg, "got window sucessfully - %lx\n", id);
	printf("got window sucessfully - %lx\n", id);

	cfg->term_window = id;

	return 0;
}

int sf_resize_process(struct sf_config *cfg, struct sniffy_msg_ws *ws)
{
	SF_LOG_PRINT2(cfg, "got ws %d %d\n", ws->ws_row, ws->ws_col);
	XResizeWindow(cfg->display, cfg->term_window, ws->ws_xpixel, ws->ws_ypixel);
	XSync(cfg->display, 0);
	return 0;
}

int sf_resize_close(struct sf_config *cfg)
{
	/* TODO close display, probably not needed, 
		since we are going down anyway */
	return 0;
}
